@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2015 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "common.h"
 #include "logfiles.h"
 #include "log.h"
+#include "active.h"
 
 #if defined(_WINDOWS)
 #	include "gnuregex.h"
@@ -32,6 +33,7 @@
 #define ZBX_SAME_FILE_NO	0
 #define ZBX_SAME_FILE_YES	1
 #define ZBX_SAME_FILE_RETRY	2
+#define ZBX_NO_FILE_ERROR	3
 
 /******************************************************************************
  *                                                                            *
@@ -58,11 +60,6 @@ static int	split_string(const char *str, const char *del, char **part1, char **p
 	size_t		str_length = 0, part1_length = 0, part2_length = 0;
 	int		ret = FAIL;
 
-	assert(NULL != str && '\0' != *str);
-	assert(NULL != del && '\0' != *del);
-	assert(NULL != part1 && '\0' == *part1);	/* target 1 must be empty */
-	assert(NULL != part2 && '\0' == *part2);	/* target 2 must be empty */
-
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s() str:'%s' del:'%s'", __function_name, str, del);
 
 	str_length = strlen(str);
@@ -276,9 +273,9 @@ static int	file_start_md5(int f, int length, md5_byte_t *md5buf, const char *fil
 		return FAIL;
 	}
 
-	md5_init(&state);
-	md5_append(&state, (const md5_byte_t *)buf, length);
-	md5_finish(&state, md5buf);
+	zbx_md5_init(&state);
+	zbx_md5_append(&state, (const md5_byte_t *)buf, length);
+	zbx_md5_finish(&state, md5buf);
 
 	return SUCCEED;
 }
@@ -490,26 +487,26 @@ static int	is_same_file(const struct st_logfile *old, const struct st_logfile *n
 	{
 		if (old->ino_lo != new->ino_lo || old->dev != new->dev)
 		{
-			/* File's inode and device id cannot differ. */
+			/* file inode and device id cannot differ */
 			goto out;
 		}
 	}
 
 	if (2 == use_ino && old->ino_hi != new->ino_hi)
 	{
-		/* File's inode (older 64-bits) cannot differ. */
+		/* file inode (older 64-bits) cannot differ */
 		goto out;
 	}
 
 	if (old->mtime > new->mtime)
 	{
-		/* File's mtime cannot decrease unless manipulated. */
+		/* file mtime cannot decrease unless manipulated */
 		goto out;
 	}
 
 	if (old->size > new->size)
 	{
-		/* File's size cannot decrease. Truncating or replacing a file with a smaller one */
+		/* File size cannot decrease. Truncating or replacing a file with a smaller one */
 		/* counts as 2 different files. */
 		goto out;
 	}
@@ -547,7 +544,7 @@ static int	is_same_file(const struct st_logfile *old, const struct st_logfile *n
 
 	if (old->md5size > new->md5size)
 	{
-		/* File's initial block size from which MD5 sum is calculated cannot decrease. */
+		/* file initial block size from which MD5 sum is calculated cannot decrease */
 		goto out;
 	}
 
@@ -656,7 +653,7 @@ static int	setup_old2new(char *old2new, struct st_logfile *old, int num_old,
 					break;
 				case ZBX_SAME_FILE_RETRY:
 					old[i].retry = 1;
-					/* break; is not missing here */
+					return FAIL;
 				case ZBX_SAME_FILE_ERROR:
 					return FAIL;
 			}
@@ -1175,10 +1172,10 @@ static void	destroy_logfile_list(struct st_logfile **logfiles, int *logfiles_all
 static void	pick_logfile(const char *directory, const char *filename, int mtime, const regex_t *re,
 		struct st_logfile **logfiles, int *logfiles_alloc, int *logfiles_num)
 {
-	char		*logfile_candidate = NULL;
+	char		*logfile_candidate;
 	zbx_stat_t	file_buf;
 
-	logfile_candidate = zbx_dsprintf(logfile_candidate, "%s%s", directory, filename);
+	logfile_candidate = zbx_dsprintf(NULL, "%s%s", directory, filename);
 
 	if (0 == zbx_stat(logfile_candidate, &file_buf))
 	{
@@ -1303,32 +1300,34 @@ clean:
  *          parameter                                                         *
  *                                                                            *
  * Parameters:                                                                *
- *     is_logrt       - [IN] Item type: 0 - log[], 1 - logrt[]                *
+ *     flags          - [IN] metric flags to check item type: log or logrt    *
  *     filename       - [IN] logfile name (regular expression with a path)    *
  *     mtime          - [IN] last modification time of the file               *
  *     logfiles       - [IN/OUT] pointer to the list of logfiles              *
  *     logfiles_alloc - [IN/OUT] number of logfiles memory was allocated for  *
  *     logfiles_num   - [IN/OUT] number of already inserted logfiles          *
  *     use_ino        - [IN/OUT] how to use inode numbers                     *
- *     err_msg        - [IN/OUT] error message why an item became             *
- *                      NOTSUPPORTED                                          *
+ *     err_msg        - [IN/OUT] error message (if FAIL or ZBX_NO_FILE_ERROR  *
+ *                      is returned)                                          *
  *                                                                            *
- * Return value: SUCCEED or FAIL                                              *
+ * Return value: SUCCEED - file list successfully built,                      *
+ *               ZBX_NO_FILE_ERROR - file(s) do not exist,                    *
+ *               FAIL - other errors                                          *
  *                                                                            *
  ******************************************************************************/
-static int	make_logfile_list(int is_logrt, const char *filename, const int *mtime, struct st_logfile **logfiles,
-		int *logfiles_alloc, int *logfiles_num, int *use_ino, char **err_msg)
+static int	make_logfile_list(unsigned char flags, const char *filename, const int *mtime,
+		struct st_logfile **logfiles, int *logfiles_alloc, int *logfiles_num, int *use_ino, char **err_msg)
 {
 	int		ret = SUCCEED, i;
 	zbx_stat_t	file_buf;
 
-	if (0 == is_logrt)	/* log[] item */
+	if (0 != (ZBX_METRIC_FLAG_LOG_LOG & flags))	/* log[] item */
 	{
 		if (0 != zbx_stat(filename, &file_buf))
 		{
 			*err_msg = zbx_dsprintf(*err_msg, "Cannot obtain information for file \"%s\": %s", filename,
 					zbx_strerror(errno));
-			ret = FAIL;
+			ret = ZBX_NO_FILE_ERROR;
 			goto clean;
 		}
 
@@ -1351,7 +1350,7 @@ static int	make_logfile_list(int is_logrt, const char *filename, const int *mtim
 		*use_ino = 1;
 #endif
 	}
-	else	/* logrt[] item */
+	else if (0 != (ZBX_METRIC_FLAG_LOG_LOGRT & flags))	/* logrt[] item */
 	{
 		char	*directory = NULL, *format = NULL;
 		int	reg_error;
@@ -1392,8 +1391,10 @@ static int	make_logfile_list(int is_logrt, const char *filename, const int *mtim
 			/* Do not make a logrt[] item NOTSUPPORTED if there are no matching log files or they are not */
 			/* accessible (can happen during a rotation), just log the problem. */
 #ifdef _WINDOWS
-			zabbix_log(LOG_LEVEL_WARNING, "there are no files matching \"%s\" in \"%s\" or insufficient "
-					"access rights", format, directory);
+			zabbix_log(LOG_LEVEL_WARNING, "there are no recently modified files matching \"%s\" in \"%s\"",
+					format, directory);
+
+			ret = ZBX_NO_FILE_ERROR;
 #else
 			if (0 != access(directory, X_OK))
 			{
@@ -1402,8 +1403,9 @@ static int	make_logfile_list(int is_logrt, const char *filename, const int *mtim
 			}
 			else
 			{
-				zabbix_log(LOG_LEVEL_WARNING, "there are no files matching \"%s\" in \"%s\"", format,
-						directory);
+				zabbix_log(LOG_LEVEL_WARNING, "there are no recently modified files matching \"%s\" in"
+						" \"%s\"", format, directory);
+				ret = ZBX_NO_FILE_ERROR;
 			}
 #endif
 		}
@@ -1413,9 +1415,11 @@ clean1:
 		zbx_free(directory);
 		zbx_free(format);
 
-		if (FAIL == ret)
+		if (FAIL == ret || ZBX_NO_FILE_ERROR == ret)
 			goto clean;
 	}
+	else
+		THIS_SHOULD_NEVER_HAPPEN;
 
 	/* Fill in MD5 sums and file indexes in the logfile list. */
 	/* These operations require opening of file, therefore we group them together. */
@@ -1455,7 +1459,7 @@ clean3:
 		}
 	}
 clean:
-	if (FAIL == ret && NULL != *logfiles)
+	if ((FAIL == ret || ZBX_NO_FILE_ERROR == ret) && NULL != *logfiles)
 		destroy_logfile_list(logfiles, logfiles_alloc, logfiles_num);
 
 	return	ret;
@@ -1520,10 +1524,11 @@ static char	*buf_find_newline(char *p, char **p_next, const char *p_end, const c
 	}
 }
 
-static int	zbx_read2(int fd, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec, int *incomplete, char **err_msg,
-		const char *encoding, zbx_vector_ptr_t *regexps, const char *pattern, const char *output_template,
-		int *p_count, int *s_count, zbx_process_value_func_t process_value, const char *server,
-		unsigned short port, const char *hostname, const char *key)
+static int	zbx_read2(int fd, unsigned char flags, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec,
+		int *incomplete, char **err_msg, const char *encoding, zbx_vector_ptr_t *regexps, const char *pattern,
+		const char *output_template, int *p_count, int *s_count, zbx_process_value_func_t process_value,
+		const char *server, unsigned short port, const char *hostname, const char *key,
+		zbx_uint64_t *lastlogsize_sent, int *mtime_sent)
 {
 	ZBX_THREAD_LOCAL static char	*buf = NULL;
 
@@ -1535,13 +1540,12 @@ static int	zbx_read2(int fd, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec
 	int				send_err;
 	zbx_uint64_t			lastlogsize1;
 
-#define BUF_SIZE	(256 * ZBX_KIBIBYTE)	/* The longest encodings use 4-bytes for every character. To send */
-						/* up to 64 k characters to the Zabbix server a 256 kB buffer might */
-						/* be required. */
+#define BUF_SIZE	(256 * ZBX_KIBIBYTE)	/* The longest encodings use 4 bytes for every character. To send */
+						/* up to 64 k characters to Zabbix server a 256 kB buffer might be */
+						/* required. */
+
 	if (NULL == buf)
-	{
 		buf = zbx_malloc(buf, (size_t)(BUF_SIZE + 1));
-	}
 
 	find_cr_lf_szbyte(encoding, &cr, &lf, &szbyte);
 
@@ -1609,7 +1613,7 @@ static int	zbx_read2(int fd, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec
 					/* regexp now (our buffer length corresponds to what we can save in the */
 					/* database). */
 
-					char	*value = NULL;
+					char	*value;
 
 					buf[BUF_SIZE] = '\0';
 
@@ -1624,25 +1628,34 @@ static int	zbx_read2(int fd, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec
 							" is running.", value);
 
 					lastlogsize1 = (size_t)offset + (size_t)nbytes;
+
 					send_err = SUCCEED;
 
 					if (SUCCEED == regexp_sub_ex(regexps, value, pattern, ZBX_CASE_SENSITIVE,
 							output_template, &item_value))
 					{
-						send_err = process_value(server, port, hostname, key, item_value, ITEM_STATE_NORMAL,
-								&lastlogsize1, mtime, NULL, NULL, NULL, NULL, 1);
+						send_err = process_value(server, port, hostname, key, item_value,
+								ITEM_STATE_NORMAL, &lastlogsize1, mtime, NULL, NULL,
+								NULL, NULL, flags | ZBX_METRIC_FLAG_PERSISTENT);
 
 						zbx_free(item_value);
 
 						if (SUCCEED == send_err)
+						{
+							*lastlogsize_sent = lastlogsize1;
+							if (NULL != mtime_sent)
+								*mtime_sent = *mtime;
+
 							(*s_count)--;
+						}
 					}
+
 					(*p_count)--;
 
 					if (SUCCEED == send_err)
 					{
 						*lastlogsize = lastlogsize1;
-						*big_rec = 1;		/* ignore the rest of this record */
+						*big_rec = 1;	/* ignore the rest of this record */
 					}
 
 					if ('\0' != *encoding)
@@ -1673,7 +1686,7 @@ static int	zbx_read2(int fd, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec
 
 				if (0 == *big_rec)
 				{
-					char	*value = NULL;
+					char	*value;
 
 					*p_nl = '\0';
 
@@ -1683,19 +1696,28 @@ static int	zbx_read2(int fd, zbx_uint64_t *lastlogsize, int *mtime, int *big_rec
 						value = p_start;
 
 					lastlogsize1 = (size_t)offset + (size_t)(p_next - buf);
+
 					send_err = SUCCEED;
 
 					if (SUCCEED == regexp_sub_ex(regexps, value, pattern, ZBX_CASE_SENSITIVE,
 							output_template, &item_value))
 					{
-						send_err = process_value(server, port, hostname, key, item_value, ITEM_STATE_NORMAL,
-								&lastlogsize1, mtime, NULL, NULL, NULL, NULL, 1);
+						send_err = process_value(server, port, hostname, key, item_value,
+								ITEM_STATE_NORMAL, &lastlogsize1, mtime, NULL, NULL,
+								NULL, NULL, flags | ZBX_METRIC_FLAG_PERSISTENT);
 
 						zbx_free(item_value);
 
 						if (SUCCEED == send_err)
+						{
+							*lastlogsize_sent = lastlogsize1;
+							if (NULL != mtime_sent)
+								*mtime_sent = *mtime;
+
 							(*s_count)--;
+						}
 					}
+
 					(*p_count)--;
 
 					if (SUCCEED == send_err)
@@ -1751,6 +1773,7 @@ out:
  *          records to Zabbix server                                          *
  *                                                                            *
  * Parameters:                                                                *
+ *     flags           - [IN] metric flags to check item type: log or logrt   *
  *     filename        - [IN] logfile name                                    *
  *     lastlogsize     - [IN/OUT] offset from the beginning of the file       *
  *     mtime           - [IN] file modification time for reporting to server  *
@@ -1764,19 +1787,10 @@ out:
  *     err_msg         - [IN/OUT] error message why an item became            *
  *                       NOTSUPPORTED                                         *
  *     encoding        - [IN] text string describing encoding.                *
- *                         The following encodings are recognized:            *
- *                           "UNICODE"                                        *
- *                           "UNICODEBIG"                                     *
- *                           "UNICODEFFFE"                                    *
- *                           "UNICODELITTLE"                                  *
- *                           "UTF-16"   "UTF16"                               *
- *                           "UTF-16BE" "UTF16BE"                             *
- *                           "UTF-16LE" "UTF16LE"                             *
- *                           "UTF-32"   "UTF32"                               *
- *                           "UTF-32BE" "UTF32BE"                             *
- *                           "UTF-32LE" "UTF32LE".                            *
- *                           "" (empty string) means a single-byte character  *
- *                           set (e.g. ASCII).                                *
+ *                       See function find_cr_lf_szbyte() for supported       *
+ *                       encodings.                                           *
+ *                       "" (empty string) means a single-byte character set  *
+ *                       (e.g. ASCII).                                        *
  *     regexps         - [IN] array of regexps                                *
  *     pattern         - [IN] pattern to match                                *
  *     output_template - [IN] output formatting template                      *
@@ -1797,11 +1811,11 @@ out:
  *           This function does not deal with log file rotation.              *
  *                                                                            *
  ******************************************************************************/
-static int	process_log(char *filename, zbx_uint64_t *lastlogsize, int *mtime, unsigned char *skip_old_data,
-		int *big_rec, int *incomplete, char **err_msg, const char *encoding, zbx_vector_ptr_t *regexps,
-		const char *pattern, const char *output_template, int *p_count, int *s_count,
-		zbx_process_value_func_t process_value, const char *server, unsigned short port, const char *hostname,
-		const char *key)
+static int	process_log(unsigned char flags, const char *filename, zbx_uint64_t *lastlogsize, int *mtime,
+		zbx_uint64_t *lastlogsize_sent, int *mtime_sent, unsigned char *skip_old_data, int *big_rec,
+		int *incomplete, char **err_msg, const char *encoding, zbx_vector_ptr_t *regexps, const char *pattern,
+		const char *output_template, int *p_count, int *s_count, zbx_process_value_func_t process_value,
+		const char *server, unsigned short port, const char *hostname, const char *key)
 {
 	const char	*__function_name = "process_log";
 
@@ -1824,8 +1838,12 @@ static int	process_log(char *filename, zbx_uint64_t *lastlogsize, int *mtime, un
 
 	if ((zbx_uint64_t)buf.st_size == *lastlogsize)
 	{
-		/* The file size has not changed. Nothing to do. Here we do not deal with a case of changing */
+		/* The file size has not changed, no new lines. Here we do not deal with a case of changing */
 		/* a logfile's content while keeping the same length. */
+
+		if (1 == *skip_old_data)
+			*skip_old_data = 0;
+
 		ret = SUCCEED;
 		goto out;
 	}
@@ -1845,7 +1863,7 @@ static int	process_log(char *filename, zbx_uint64_t *lastlogsize, int *mtime, un
 				filename, l_size);
 	}
 
-	if ((zbx_uint64_t)buf.st_size < l_size)		/* handle file truncation */
+	if ((zbx_uint64_t)buf.st_size < l_size)	/* handle file truncation */
 		l_size = 0;
 
 	if ((zbx_offset_t)-1 != zbx_lseek(f, l_size, SEEK_SET))
@@ -1853,8 +1871,9 @@ static int	process_log(char *filename, zbx_uint64_t *lastlogsize, int *mtime, un
 		*lastlogsize = l_size;
 		*skip_old_data = 0;
 
-		ret = zbx_read2(f, lastlogsize, mtime, big_rec, incomplete, err_msg, encoding, regexps, pattern,
-				output_template, p_count, s_count, process_value, server, port, hostname, key);
+		ret = zbx_read2(f, flags, lastlogsize, mtime, big_rec, incomplete, err_msg, encoding, regexps, pattern,
+				output_template, p_count, s_count, process_value, server, port, hostname, key,
+				lastlogsize_sent, mtime_sent);
 	}
 	else
 	{
@@ -1881,34 +1900,26 @@ out:
  * Purpose: Find new records in logfiles                                      *
  *                                                                            *
  * Parameters:                                                                *
- *     is_logrt         - [IN] Item type: 0 - log[], 1 - logrt[]              *
+ *     flags            - [IN] metric flags to check item type: log or logrt  *
  *     filename         - [IN] logfile name (regular expression with a path)  *
  *     lastlogsize      - [IN/OUT] offset from the beginning of the file      *
  *     mtime            - [IN/OUT] last modification time of the file         *
+ *     lastlogsize_sent - [OUT] lastlogsize value that was last sent          *
+ *     mtime_sent       - [OUT] mtime value that was last sent                *
  *     skip_old_data    - [IN/OUT] start from the beginning of the file or    *
  *                        jump to the end                                     *
  *     big_rec          - [IN/OUT] state variable to remember whether a long  *
  *                        record is being processed                           *
  *     use_ino          - [IN/OUT] how to use inode numbers                   *
- *     error_count      - [IN/OUT] number of errors (for limiting retries)    *
  *     err_msg          - [IN/OUT] error message why an item became           *
  *                        NOTSUPPORTED                                        *
  *     logfiles_old     - [IN/OUT] array of logfiles from the last check      *
  *     logfiles_num_old - [IN/OUT] number of elements in "logfiles_old"       *
  *     encoding         - [IN] text string describing encoding.               *
- *                          The following encodings are recognized:           *
- *                            "UNICODE"                                       *
- *                            "UNICODEBIG"                                    *
- *                            "UNICODEFFFE"                                   *
- *                            "UNICODELITTLE"                                 *
- *                            "UTF-16"   "UTF16"                              *
- *                            "UTF-16BE" "UTF16BE"                            *
- *                            "UTF-16LE" "UTF16LE"                            *
- *                            "UTF-32"   "UTF32"                              *
- *                            "UTF-32BE" "UTF32BE"                            *
- *                            "UTF-32LE" "UTF32LE".                           *
- *                          "" (empty string) means a single-byte character   *
- *                             set.                                           *
+ *                        See function find_cr_lf_szbyte() for supported      *
+ *                        encodings.                                          *
+ *                        "" (empty string) means a single-byte character set *
+ *                        (e.g. ASCII).                                       *
  *     regexps          - [IN] array of regexps                               *
  *     pattern          - [IN] pattern to match                               *
  *     output_template  - [IN] output formatting template                     *
@@ -1926,21 +1937,22 @@ out:
  * Author: Dmitry Borovikov (logrotation)                                     *
  *                                                                            *
  ******************************************************************************/
-int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *mtime, unsigned char *skip_old_data,
-		int *big_rec, int *use_ino, int *error_count, char **err_msg, struct st_logfile **logfiles_old,
-		int *logfiles_num_old, const char *encoding, zbx_vector_ptr_t *regexps, const char *pattern,
-		const char *output_template, int *p_count, int *s_count, zbx_process_value_func_t process_value,
-		const char *server, unsigned short port, const char *hostname, const char *key)
+int	process_logrt(unsigned char flags, const char *filename, zbx_uint64_t *lastlogsize, int *mtime,
+		zbx_uint64_t *lastlogsize_sent, int *mtime_sent, unsigned char *skip_old_data, int *big_rec,
+		int *use_ino, char **err_msg, struct st_logfile **logfiles_old, int *logfiles_num_old,
+		const char *encoding, zbx_vector_ptr_t *regexps, const char *pattern, const char *output_template,
+		int *p_count, int *s_count, zbx_process_value_func_t process_value, const char *server,
+		unsigned short port, const char *hostname, const char *key)
 {
 	const char		*__function_name = "process_logrt";
 	int			i, j, start_idx, ret = FAIL, logfiles_num = 0, logfiles_alloc = 0, seq = 1,
-				max_old_seq = 0, old_last, from_first_file = 1;
+				max_old_seq = 0, old_last, from_first_file = 1, res;
 	char			*old2new = NULL;
 	struct st_logfile	*logfiles = NULL;
 	time_t			now;
 
-	zabbix_log(LOG_LEVEL_DEBUG, "In %s() is_logrt:%d filename:'%s' lastlogsize:" ZBX_FS_UI64 " mtime:%d "
-			"error_count:%d", __function_name, is_logrt, filename, *lastlogsize, *mtime, *error_count);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s() is_logrt:%d filename:'%s' lastlogsize:" ZBX_FS_UI64 " mtime:%d",
+			__function_name, ZBX_METRIC_FLAG_LOG_LOGRT & flags, filename, *lastlogsize, *mtime);
 
 	/* Minimize data loss if the system clock has been set back in time. */
 	/* Setting the clock ahead of time is harmless in our case. */
@@ -1955,13 +1967,33 @@ int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *
 				"seconds back.", (int)(old_mtime - now));
 	}
 
-	if (SUCCEED != make_logfile_list(is_logrt, filename, mtime, &logfiles, &logfiles_alloc, &logfiles_num, use_ino,
-			err_msg))
+	if (SUCCEED != (res = make_logfile_list(flags, filename, mtime, &logfiles, &logfiles_alloc, &logfiles_num,
+			use_ino, err_msg)))
 	{
-		/* an error occurred or a file was not accessible for a log[] item */
-		(*error_count)++;
-		ret = SUCCEED;
-		goto out;
+		if (ZBX_NO_FILE_ERROR == res)
+		{
+			if (1 == *skip_old_data)
+			{
+				*skip_old_data = 0;
+
+				zabbix_log(LOG_LEVEL_DEBUG, "%s(): no files, setting skip_old_data to 0",
+						__function_name);
+			}
+
+			if (0 != (ZBX_METRIC_FLAG_LOG_LOGRT & flags) && 0 == *logfiles_num_old)
+			{
+				/* Both the old and the new log file lists are empty. That means the agent has not */
+				/* seen any log files for this logrt[] item since started. If log files appear later */
+				/* then analyze them from start, do not apply the 'lastlogsize' received from server */
+				/* anymore. */
+
+				*lastlogsize = 0;
+			}
+		}
+
+		/* file was not accessible for a log[] item or an error occurred */
+		if (0 != (ZBX_METRIC_FLAG_LOG_LOG & flags) || (0 != (ZBX_METRIC_FLAG_LOG_LOGRT & flags) && FAIL == res))
+			goto out;
 	}
 
 	if (0 == logfiles_num)
@@ -1990,8 +2022,6 @@ int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *
 		{
 			destroy_logfile_list(&logfiles, &logfiles_alloc, &logfiles_num);
 			zbx_free(old2new);
-			(*error_count)++;
-			ret = SUCCEED;
 			goto out;
 		}
 
@@ -2055,14 +2085,14 @@ int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *
 
 	if (SUCCEED == zabbix_check_log_level(LOG_LEVEL_DEBUG))
 	{
-		zabbix_log(LOG_LEVEL_DEBUG, "process_logrt() old file list:");
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() old file list:", __function_name);
 		if (NULL != *logfiles_old)
 			print_logfile_list(*logfiles_old, *logfiles_num_old);
 		else
 			zabbix_log(LOG_LEVEL_DEBUG, "   file list empty");
 
-		zabbix_log(LOG_LEVEL_DEBUG, "process_logrt() new file list: (mtime:%d lastlogsize:" ZBX_FS_UI64
-				" start_idx:%d)", *mtime, *lastlogsize, start_idx);
+		zabbix_log(LOG_LEVEL_DEBUG, "%s() new file list: (mtime:%d lastlogsize:" ZBX_FS_UI64
+				" start_idx:%d)", __function_name, *mtime, *lastlogsize, start_idx);
 		if (NULL != logfiles)
 			print_logfile_list(logfiles, logfiles_num);
 		else
@@ -2093,10 +2123,11 @@ int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *
 			if (start_idx != i)
 				*lastlogsize = logfiles[i].processed_size;
 
-			ret = process_log(logfiles[i].filename, lastlogsize, (1 == is_logrt) ? mtime : NULL,
-					skip_old_data, big_rec, &logfiles[i].incomplete, err_msg, encoding, regexps,
-					pattern, output_template, p_count, s_count, process_value, server, port,
-					hostname, key);
+			ret = process_log(flags, logfiles[i].filename, lastlogsize,
+					(0 != (ZBX_METRIC_FLAG_LOG_LOGRT & flags) ? mtime : NULL), lastlogsize_sent,
+					(0 != (ZBX_METRIC_FLAG_LOG_LOGRT & flags) ? mtime_sent : NULL), skip_old_data,
+					big_rec, &logfiles[i].incomplete, err_msg, encoding, regexps, pattern,
+					output_template, p_count, s_count, process_value, server, port, hostname, key);
 
 			/* process_log() advances 'lastlogsize' only on success therefore */
 			/* we do not check for errors here */
@@ -2108,11 +2139,7 @@ int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *
 			logfiles[i].seq = seq++;
 
 			if (SUCCEED != ret)
-			{
-				(*error_count)++;
-				ret = SUCCEED;
 				break;
-			}
 
 			if (0 >= *p_count || 0 >= *s_count)
 			{
@@ -2140,8 +2167,7 @@ int	process_logrt(int is_logrt, char *filename, zbx_uint64_t *lastlogsize, int *
 	if (0 < logfiles_num)
 		*logfiles_old = logfiles;
 out:
-	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s error_count:%d", __function_name, zbx_result_string(ret),
-			*error_count);
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
 
 	return ret;
 }
