@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -479,9 +479,6 @@ static struct snmp_session	*zbx_snmp_open_session(const DC_ITEM *item, char *err
 #endif
 	session.peername = addr;
 
-	/* remote_port is no longer used in latest versions of Net-SNMP */
-	session.remote_port = item->interface.port;
-
 	if (SNMP_VERSION_1 == session.version || SNMP_VERSION_2c == session.version)
 	{
 		session.community = (u_char *)item->snmp_community;
@@ -684,6 +681,10 @@ static char	*zbx_snmp_get_octet_string(const struct variable_list *var)
 	{
 		strval_dyn = zbx_strdup(strval_dyn, buffer + 5);
 	}
+	else if (0 == strncmp(buffer, "BITS: ", 6))
+	{
+		strval_dyn = zbx_strdup(strval_dyn, buffer + 6);
+	}
 	else
 	{
 		/* snprint_value() escapes hintless ASCII strings, so */
@@ -2151,7 +2152,15 @@ out:
 
 void	zbx_init_snmp(void)
 {
+	sigset_t	mask, orig_mask;
+
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGTERM);
+	sigprocmask(SIG_BLOCK, &mask, &orig_mask);
+
 	init_snmp(progname);
+
+	sigprocmask(SIG_SETMASK, &orig_mask, NULL);
 }
 
 #endif	/* HAVE_NETSNMP */
